Attribute VB_Name = "modMouseHook"
Option Explicit

Public Enum MouseType
    Left = 0
    Right = 1
    Others = 2
End Enum

Private Type POINTAPI
    x As Long
    y As Long
End Type

Private Type MOUSEHOOKSTRUCT
    pt              As POINTAPI
    hwnd            As Long
    wHitTestCode    As Long
    dwExtraInfo     As Long
End Type

Private Declare Function GetSystemMetrics Lib "user32.dll" ( _
                ByVal nIndex As Long) As Long
Private Declare Function SetWindowsHookEx Lib "user32" Alias "SetWindowsHookExA" ( _
                ByVal idHook As Long, _
                ByVal lpfn As Long, _
                ByVal hmod As Long, _
                ByVal dwThreadId As Long) As Long
Private Declare Function UnhookWindowsHookEx Lib "user32" ( _
                ByVal hHook As Long) As Long
Private Declare Function CallNextHookEx Lib "user32" ( _
                ByVal hHook As Long, _
                ByVal nCode As Long, _
                ByVal wParam As Long, _
                ByVal lParam As Long) As Long
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" ( _
                pDest As Any, _
                pSource As Any, _
                ByVal cb As Long)
Private Declare Function GetDoubleClickTime Lib "user32.dll" () As Long
Private Declare Function GetTickCount Lib "kernel32.dll" () As Long
Private Declare Function SetTimer Lib "user32.dll" ( _
                ByVal hwnd As Long, _
                ByVal nIDEvent As Long, _
                ByVal uElapse As Long, _
                ByVal lpTimerFunc As Long) As Long
Private Declare Function KillTimer Lib "user32.dll" ( _
                ByVal hwnd As Long, _
                ByVal nIDEvent As Long) As Long

Private Const SM_CXDOUBLECLK    As Long = 36
Private Const SM_CYDOUBLECLK    As Long = 37
Private Const WM_LBUTTONDOWN    As Long = &H201 '513
Private Const WM_LBUTTONUP      As Long = &H202 '514
Private Const WM_RBUTTONDOWN    As Long = &H204 '516
Private Const WM_RBUTTONUP      As Long = &H205 '517
Private Const WM_MOUSEMOVE      As Long = &H200
Private Const WH_MOUSE_LL       As Long = 14
Private Const HC_ACTION         As Long = 0&
Private m_Hook                  As Long
Public lngDoubleClickTime       As Long
Public lngDoubleClickX          As Long
Public lngDoubleClickY          As Long
Private TimerOwnerHandle        As Long
Private bolLeftClicked          As Boolean
Private bolFirstCriteriaMet     As Boolean
Private bolSecondCriteriaMet    As Boolean
Private bolThirdCriteriaMet     As Boolean
Private bolFinishLeftClick      As Boolean
Private bolFinishRightClick     As Boolean
Private CTest                   As clsDoubleClick

Public Sub Main()
    Set CTest = New clsDoubleClick
    TimerOwnerHandle = Form1.hwnd
    lngDoubleClickX = GetSystemMetrics(SM_CXDOUBLECLK)
    lngDoubleClickY = GetSystemMetrics(SM_CYDOUBLECLK)
    lngDoubleClickTime = GetDoubleClickTime
    Debug.Print lngDoubleClickTime
    Form1.Show
End Sub

Public Sub TimerProc(ByVal hwnd As Long, ByVal uMsg As Long, ByVal idEvent As Long, ByVal dwTime As Long)
    ResetCriteria
    StopTimer
End Sub

Public Sub HookMouse()
    m_Hook = SetWindowsHookEx(WH_MOUSE_LL, AddressOf LowLevelMouseProc, App.hInstance, 0&)
End Sub

Public Sub UnHookMouse()
    UnhookWindowsHookEx m_Hook
End Sub

Public Function LowLevelMouseProc(ByVal nCode As Long, _
                                  ByVal wParam As Long, _
                                  ByVal lParam As Long) As Long

    Dim MouseData As MOUSEHOOKSTRUCT
    
    If nCode = HC_ACTION Then
    
        If bolFirstCriteriaMet = False Then 'LEFT DOUBLE CLICK
            
            Select Case wParam
            Case WM_LBUTTONDOWN
                
                CopyMemory MouseData, ByVal lParam, Len(MouseData)
                If CTest.IsDoubleClick(Left, MouseData.pt.x, MouseData.pt.y, GetTickCount) = True Then
                    bolFinishLeftClick = True
                End If
                
            Case WM_LBUTTONUP
                
                'reset timer
                StopTimer
                StartTimer
                If bolFinishLeftClick = True Then
                    bolFirstCriteriaMet = True
                    bolFinishLeftClick = False
                End If
                
            Case WM_MOUSEMOVE 'ignore mouse move
                
            Case Else 'reset for other mouse input
                ResetCriteria
            End Select
            
        ElseIf bolSecondCriteriaMet = False Then 'RIGHT CLICK
            
            Select Case wParam
            Case WM_RBUTTONDOWN
            
                bolFinishRightClick = True
                
            Case WM_RBUTTONUP
            
                'reset timer
                StopTimer
                StartTimer
                If bolFinishRightClick = True Then
                    bolSecondCriteriaMet = True
                    Set CTest = New clsDoubleClick
                    bolFinishRightClick = False
                End If
                
            Case WM_MOUSEMOVE 'ignore mouse move
                
            Case Else 'reset for other mouse input
            
                ResetCriteria
                
            End Select
            
        ElseIf bolThirdCriteriaMet = False Then 'LEFT DOUBLE CLICK
            
            Select Case wParam
            Case WM_LBUTTONDOWN
                
                CopyMemory MouseData, ByVal lParam, Len(MouseData)
                If CTest.IsDoubleClick(Left, MouseData.pt.x, MouseData.pt.y, GetTickCount) = True Then
                    bolFinishLeftClick = True
                End If
                
            Case WM_LBUTTONUP 'ignore mouse up since it comes from here
                
                'reset timer
                StopTimer
                StartTimer
                If bolFinishLeftClick = True Then
                    Debug.Print "Hotkey Successful"
                    Form1.Print "Hotkey Successful"
                    Beep
                    ResetCriteria
                End If
                
            Case WM_MOUSEMOVE 'ignore mouse move
                
            Case Else 'reset for other mouse input
                
                ResetCriteria
                
            End Select
            
        End If
    End If
    
    LowLevelMouseProc = CallNextHookEx(m_Hook, nCode, wParam, lParam)
End Function

Private Sub ResetCriteria()
    StopTimer
    Set CTest = New clsDoubleClick
    bolFirstCriteriaMet = False
    bolSecondCriteriaMet = False
    bolThirdCriteriaMet = False
    bolLeftClicked = False
    bolFinishLeftClick = False
    bolFinishRightClick = False
End Sub

Public Sub StartTimer()
    Dim retval As Long  ' return value

    retval = SetTimer(TimerOwnerHandle, 1, lngDoubleClickTime, AddressOf TimerProc)
End Sub

Public Sub StopTimer()
    Dim retval As Long  ' return value

    retval = KillTimer(TimerOwnerHandle, 1)
End Sub

